@@ -89,6 +89,7 @@ def extract_metar(data: str) -> list:
     return data[:len(data)-1]
 
 def parse_metar(message: str, year: int, month: int) -> dict:
+    message = message.strip()
     print(f"Parsing message: {message}")
     icaoID = re_ID.match(message).group('id')
     if deu.isUSIdentifier(icaoID):
@@ -96,6 +97,9 @@ def parse_metar(message: str, year: int, month: int) -> dict:
     else:
         decoded = annex3(message)
 
+    for key in decoded:
+        print(key, ': ', decoded[key])
+
     output = deepcopy(metar_template)
 
     output['report_type'] = message[0:5]
@@ -117,7 +121,7 @@ def parse_metar(message: str, year: int, month: int) -> dict:
         #         # print(f'Mapping {icao} to {tsi}')
         #         break
 
-        try:
+        try: 
             output['tsi'] = icao
             # output['block_no'] = tsi[0:2]
             # output['station_no'] = tsi[2:5]
@@ -142,14 +146,19 @@ def parse_metar(message: str, year: int, month: int) -> dict:
         wind_uom = decoded['wind']['uom']
         wind_speed = decoded['wind']['ff']
         #   this bufr template records wind speed in m/s
-        #   if wind speed reported in m/s, can copy directly to output
-        if wind_uom == 'm/s':
-            output['wind_speed'] = wind_speed
-        #   if wind speed report in knots, need to convert to m/s for bufr
-        elif wind_uom == '[kn_i]':
-            output['wind_speed'] = round(int(wind_speed)*0.514444, 2)
+
+        #   check for missing value
+        if wind_speed != '//':
+            #   if wind speed reported in m/s, can copy directly to output
+            if wind_uom == 'm/s':
+                output['wind_speed'] = wind_speed
+            #   if wind speed report in knots, need to convert to m/s for bufr
+            elif wind_uom == '[kn_i]':
+                output['wind_speed'] = round(int(wind_speed)*0.514444, 2)
         
-        output['wind_direction'] = decoded['wind']['dd']
+        #   check for missing value
+        if decoded['wind']['dd'] != '///':
+            output['wind_direction'] = decoded['wind']['dd']
 
         #   perform same unit conversions on gust speed if recorded
         if 'gg' in decoded['wind']:
@@ -165,15 +174,12 @@ def parse_metar(message: str, year: int, month: int) -> dict:
             output['variable_extreme_cw'] = int(decoded['wind']['cw'])
 
     if 'vsby' in decoded:
-        vis_uom = decoded['vsby']['uom']
-        visibility = decoded['vsby']['value']
-        #   bufr uses meters for visibility. if reported in meters, just copy to output
-        if vis_uom == 'm':
-            output['visibility'] == visibility
-        #   if reported in Statute Miles (SM), convert to m/s
-        elif vis_uom == '[mi_i]':
-            output['visibility'] == math.floor(float(visibility)*1609.34)
-        output['visibility_uom'] = decoded['vsby']['uom']
+        #   check for missing value
+        if decoded['vsby']['value'] != '////':
+            # Metar reports visibility in statute miles. Need to convert value to meters to be bufr compatible
+            vis_miles = float(decoded['vsby']['value'])
+            output['visibility'] = math.floor(vis_miles * 1609.34)
+            output['visibility_uom'] = decoded['vsby']['uom']
 
     # TODO need to incorporate visibility variation as outlined in section 15.6.2 in FM-15 manual
 
@@ -272,9 +278,14 @@ def parse_metar(message: str, year: int, month: int) -> dict:
             elif cloud_amount == 'OVC':
                 # table 020011 lists two codes that might work for OVERCAST clouds, 9 and 10. Hard coding as 10 for now
                 output[f'cloud_amount_{i+1}'] = 10
+            elif cloud_amount == '///':
+                # codes undiscernable observation or observation not made as 15
+                output[f'cloud_amount_{i+1}'] = 15
             # convert reported cloud height from hundreds of ft. to meters for bufr
-            cloud_height_ft = float(decoded['sky']['str'][i][3:6]) * 100
-            output[f'cloud_height_{i+1}'] = math.floor(cloud_height_ft*0.3048)
+            # check for missing value
+            if decoded['sky']['str'][i][3:6] != '///':
+                cloud_height_ft = float(decoded['sky']['str'][i][3:6]) * 100
+                output[f'cloud_height_{i+1}'] = math.floor(cloud_height_ft*0.3048)
 
             # TODO: capture significant convective clouds, namely TCU OR CB using ecCodes
             if len(decoded['sky']['str'][i]) > 6:
@@ -295,29 +306,27 @@ def parse_metar(message: str, year: int, month: int) -> dict:
 
     if 'temps' in decoded:
         if 'air' in decoded['temps']:
-            # metar records temperatures in Celsius. Need to convert to Kelvin for bufr
-            air_temp = float(decoded['temps']['air'])
-            output['air_temp'] = air_temp + 273.15
+            #   check for missing value
+            if decoded['temps']['air'] != '//':
+                # metar records temperatures in Celsius. Need to convert to Kelvin for bufr
+                air_temp = float(decoded['temps']['air'])
+                output['air_temp'] = air_temp + 273.15
         if 'dewpoint' in decoded['temps']:
-            # metar records temperatures in Celsius. Need to convert to Kelvin for bufr
-            dewpoint_temp = float(decoded['temps']['dewpoint'])
-            output['dew_point_temp'] = dewpoint_temp + 273.15
+            #   check for missing value
+            if decoded['temps']['dewpoint'] != '//':
+                # metar records temperatures in Celsius. Need to convert to Kelvin for bufr
+                dewpoint_temp = float(decoded['temps']['dewpoint'])
+                output['dew_point_temp'] = dewpoint_temp + 273.15
 
-    # if 'altimeter' in decoded:
-    #     altimeter = float(decoded['altimeter']['value'])
-    #     # metar records altimeter in inches of Mercury. need to convert to Pascals for bufr
-    #     output['altimeter'] = round(altimeter*3386.39, 2)
-    #     output['altimeter_uom'] = decoded['altimeter']['uom']
-    
     if 'altimeter' in decoded:
         altimeter = float(decoded['altimeter']['value'])
         alt_uom = decoded['altimeter']['uom']
-        #   if units of measurement are in hectopascals, hPa, multiply by 100 to convert to Pascals for bufr
         if alt_uom == 'hPa':
             output['altimeter'] = altimeter*100
         elif alt_uom == "[in_i'Hg]":
-            # if units of measurement is inches of Mercury. need to convert to Pascals for bufr
+            # if metar records altimeter in inches of Mercury. need to convert to Pascals for bufr
             output['altimeter'] = round(altimeter*3386.39, 2)
+        output['altimeter_uom'] = decoded['altimeter']['uom']
 
     return output, num_rvr, num_vrbrvr, num_pcp, num_obv, num_sky
 
@@ -371,7 +380,6 @@ def transform(data: str, metadata: str, year: int, month: int) -> Iterator[dict]
 
         conversion_success = {}
 
-        import pdb; pdb.set_trace()
         try:
             msg, num_rvr, num_vrbrvr, num_pcp, num_obv, num_sky = parse_metar(metar, year, month)
             tsi = msg['tsi']
@@ -384,6 +392,11 @@ def transform(data: str, metadata: str, year: int, month: int) -> Iterator[dict]
             conversion_success[tsi] = False
             LOGGER.warning(f"Station {tsi} not found in station file")
 
+            #   Question for Tom on how wigos mapping is resulting in conversion success being set even though station
+            #   is not present
+            #   suspect stale data in wsi is retained from previous iteration
+            continue
+
         # parse WSI to get sections
         try:
             wsi_series, wsi_issuer, wsi_issue_number, wsi_local = wsi.split("-")   # noqa
@@ -407,6 +420,10 @@ def transform(data: str, metadata: str, year: int, month: int) -> Iterator[dict]
             else:
                 LOGGER.warning((f"Invalid WSI ({wsi}) found in station file,"
                                 " unable to parse"))
+            #   Question for Tom on how wigos mapping is resulting in conversion success being set even though station
+            #   is not present
+            #   suspect stale data in wsi is retained from previous iteration
+            continue
 
         # TODO: update mappings based on number of precipitation observations and observed phenomena
 
@@ -466,6 +483,10 @@ def transform(data: str, metadata: str, year: int, month: int) -> Iterator[dict]
             LOGGER.error(e)
             LOGGER.error("Error creating BUFRMessage")
             conversion_success[tsi] = False
+            #   Question for Tom on how wigos mapping is resulting in conversion success being set even though station
+            #   is not present
+            #   suspect stale data in wsi is retained from previous iteration
+            continue
 
         # parse
         if conversion_success[tsi]:
@@ -475,7 +496,7 @@ def transform(data: str, metadata: str, year: int, month: int) -> Iterator[dict]
                 LOGGER.error(e)
                 LOGGER.error("Error parsing message")
                 conversion_success[tsi] = False
-            message.parse(msg, mapping)
+
         # Only convert to BUFR if there's no errors so far
         if conversion_success[tsi]:
             try:
